@@ -74,6 +74,8 @@ struct _GnomeXkbInfoPrivate
   GHashTable *layouts_by_language;
   GHashTable *layouts_table;
 
+  gboolean include_extras;
+
   /* Only used while parsing */
   XkbOptionGroup *current_parser_group;
   XkbOption *current_parser_option;
@@ -459,6 +461,8 @@ parse_end_element (GMarkupParseContext  *context,
     }
   else if (strcmp (element_name, "group") == 0)
     {
+      XkbOptionGroup *existing_group;
+
       if (!priv->current_parser_group->description || !priv->current_parser_group->id)
         {
           g_set_error (error, G_MARKUP_ERROR, G_MARKUP_ERROR_INVALID_CONTENT,
@@ -466,9 +470,30 @@ parse_end_element (GMarkupParseContext  *context,
           return;
         }
 
-      g_hash_table_replace (priv->option_groups_table,
-                            priv->current_parser_group->id,
-                            priv->current_parser_group);
+      existing_group = g_hash_table_lookup (priv->option_groups_table,
+                                            priv->current_parser_group->id);
+
+      if (existing_group)
+        {
+          GHashTableIter iter;
+          gpointer key, value;
+
+          g_hash_table_iter_init (&iter, priv->current_parser_group->options_table);
+          while (g_hash_table_iter_next (&iter, &key, &value))
+            {
+              g_hash_table_iter_steal (&iter);
+              g_hash_table_replace (existing_group->options_table, key, value);
+            }
+
+          free_option_group (priv->current_parser_group);
+        }
+      else
+        {
+          g_hash_table_replace (priv->option_groups_table,
+                                priv->current_parser_group->id,
+                                priv->current_parser_group);
+        }
+
       priv->current_parser_group = NULL;
     }
   else if (strcmp (element_name, "option") == 0)
@@ -555,8 +580,6 @@ static void
 parse_rules (GnomeXkbInfo *self)
 {
   GnomeXkbInfoPrivate *priv = self->priv;
-  GSettings *settings;
-  gboolean show_all_sources;
   gchar *file_path;
   GError *error = NULL;
 
@@ -587,11 +610,7 @@ parse_rules (GnomeXkbInfo *self)
     goto cleanup;
   g_free (file_path);
 
-  settings = g_settings_new ("org.cinnamon.desktop.input-sources");
-  show_all_sources = g_settings_get_boolean (settings, "show-all-sources");
-  g_object_unref (settings);
-
-  if (!show_all_sources)
+  if (!priv->include_extras)
     return;
 
   file_path = get_xml_rules_file_path (".extras.xml");
@@ -627,6 +646,7 @@ static void
 gnome_xkb_info_init (GnomeXkbInfo *self)
 {
   self->priv = gnome_xkb_info_get_instance_private (self);
+  self->priv->include_extras = FALSE;
 }
 
 static void
@@ -665,6 +685,22 @@ gnome_xkb_info_new (void)
   return g_object_new (GNOME_TYPE_XKB_INFO, NULL);
 }
 
+/**
+ * gnome_xkb_info_new_with_extras:
+ *
+ * Returns: (transfer full): a new #GnomeXkbInfo instance that includes extra layouts and options.
+ */
+GnomeXkbInfo *
+gnome_xkb_info_new_with_extras (void)
+{
+  GnomeXkbInfo *info;
+
+  info = g_object_new (GNOME_TYPE_XKB_INFO, NULL);
+  info->priv->include_extras = TRUE;
+
+  return info;
+}
+
 /**
  * gnome_xkb_info_get_all_layouts:
  * @self: a #GnomeXkbInfo
@@ -751,6 +787,36 @@ gnome_xkb_info_description_for_group (GnomeXkbInfo *self,
   return XKEYBOARD_CONFIG_(group->description);
 }
 
+/**
+ * gnome_xkb_info_get_option_group_allows_multiple_selection:
+ * @self: a #GnomeXkbInfo
+ * @group_id: identifier for group
+ *
+ * Gets whether multiple options can be selected for a given group.
+ *
+ * Return value: %TRUE if multiple selection is allowed, %FALSE otherwise.
+ */
+gboolean
+gnome_xkb_info_get_option_group_allows_multiple_selection (GnomeXkbInfo *self,
+                                                           const gchar  *group_id)
+{
+  GnomeXkbInfoPrivate *priv;
+  const XkbOptionGroup *group;
+
+  g_return_val_if_fail (GNOME_IS_XKB_INFO (self), FALSE);
+
+  priv = self->priv;
+
+  if (!ensure_rules_are_parsed (self))
+    return FALSE;
+
+  group = g_hash_table_lookup (priv->option_groups_table, group_id);
+  if (!group)
+    return FALSE;
+
+  return group->allow_multiple_selection;
+}
+
 /**
  * gnome_xkb_info_get_options_for_group:
  * @self: a #GnomeXkbInfo
@@ -764,7 +830,7 @@ gnome_xkb_info_description_for_group (GnomeXkbInfo *self,
  * the strings themselves, those are internally allocated and must not
  * be modified.
  *
- * Since: 3.6
+ * Since: 6.6
  */
 GList *
 gnome_xkb_info_get_options_for_group (GnomeXkbInfo *self,
